package envoy.client.ui;

import java.awt.Color;
import java.awt.ComponentOrientation;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.util.ArrayList;
import java.util.List;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextPane;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;
import javax.swing.border.EmptyBorder;

import envoy.client.Chat;
import envoy.client.Client;
import envoy.schema.Message;
import envoy.schema.User;
import envoy.schema.Users;

/**
 * Project: <strong>envoy-client</strong><br>
 * File: <strong>ChatWindow.java</strong><br>
 * Created: <strong>28 Sep 2019</strong><br>
 * 
 * @author Kai S. K. Engelbart
 * @author Maximilian K&auml;fer
 * @author Leon Hofmeister
 * @since Envoy 0.1
 */
public class ChatWindow extends JFrame {

	private static final long serialVersionUID = 6865098428255463649L;

	private long	recipientID	= 0;
	private JPanel	contentPane	= new JPanel();

	private Client client;

	private DefaultListModel<String>	messageListModel	= new DefaultListModel<>();
	private List<Chat>					partnerChatList		= new ArrayList<Chat>();

	@SuppressWarnings("null")
	public ChatWindow(Client client) {
		this.client = client;

		Users chatUsers = client.getUsersListXml();

		for (int i = 0; i < chatUsers.getUser().size(); i++) {
			Chat userChat = new Chat();
			userChat.setUserID(chatUsers.getUser().get(i).getID());
			partnerChatList.add(userChat);
		}

		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setBounds(100, 100, 600, 800);
		setTitle("Envoy");
		setLocationRelativeTo(null);

		contentPane.setBackground(new Color(0, 0, 0));
		contentPane.setForeground(Color.white);
		contentPane.setBorder(new EmptyBorder(0, 5, 0, 0));
		setContentPane(contentPane);
		GridBagLayout gbl_contentPane = new GridBagLayout();
		gbl_contentPane.columnWidths	= new int[] { 1, 1, 1 };
		gbl_contentPane.rowHeights		= new int[] { 1, 1, 1 };
		gbl_contentPane.columnWeights	= new double[] { 0.3, 1.0, 0.1 };
		gbl_contentPane.rowWeights		= new double[] { 0.05, 1, 0.07 };
		contentPane.setLayout(gbl_contentPane);

		JList<String> elementList = new JList<>();
		elementList.setFocusTraversalKeysEnabled(false);

		elementList.setSelectionForeground(new Color(255, 255, 255));
		elementList.setSelectionBackground(new Color(102, 0, 153));
		elementList.setComponentOrientation(ComponentOrientation.LEFT_TO_RIGHT);
		elementList.setForeground(new Color(255, 255, 255));
		elementList.setBackground(new Color(51, 51, 51));

		elementList.setModel(messageListModel);
		elementList.setFont(new Font("Arial", Font.PLAIN, 17));
		elementList.setFixedCellHeight(60);
		elementList.setBorder(new EmptyBorder(5, 5, 5, 5));

		JScrollPane scrollPane = new JScrollPane();
		scrollPane.setForeground(new Color(0, 0, 0));
		scrollPane.setBackground(new Color(51, 51, 51));
		scrollPane.setViewportView(elementList);
		scrollPane.setBorder(null);

		GridBagConstraints gbc_scrollPane = new GridBagConstraints();
		gbc_scrollPane.fill			= GridBagConstraints.BOTH;
		gbc_scrollPane.gridwidth	= 2;
		gbc_scrollPane.gridx		= 1;
		gbc_scrollPane.gridy		= 1;

		gbc_scrollPane.insets = new Insets(0, 10, 10, 10);

		contentPane.add(scrollPane, gbc_scrollPane);

		// Message enter field
		JTextArea messageEnterTextfield = new JTextArea();
		messageEnterTextfield.setCaretColor(new Color(255, 255, 255));
		messageEnterTextfield.setForeground(new Color(255, 255, 255));
		messageEnterTextfield.setBackground(new Color(51, 51, 51));
		messageEnterTextfield.setLineWrap(true);
		messageEnterTextfield.setBorder(null);
		messageEnterTextfield.setFont(new Font("Arial", Font.PLAIN, 17));
		messageEnterTextfield.setBorder(new EmptyBorder(5, 5, 5, 5));

		GridBagConstraints gbc_moveSelectionMessageEnterTextfield = new GridBagConstraints();
		gbc_moveSelectionMessageEnterTextfield.fill		= GridBagConstraints.BOTH;
		gbc_moveSelectionMessageEnterTextfield.gridx	= 1;
		gbc_moveSelectionMessageEnterTextfield.gridy	= 2;

		gbc_moveSelectionMessageEnterTextfield.insets = new Insets(10, 10, 10, 10);

		contentPane.add(messageEnterTextfield, gbc_moveSelectionMessageEnterTextfield);

		// Post Button
		JButton postButton = new JButton("Post");
		postButton.setForeground(new Color(255, 255, 255));
		postButton.setBackground(new Color(102, 51, 153));
		postButton.setBorderPainted(false);

		GridBagConstraints gbc_moveSelectionPostButton = new GridBagConstraints();

		gbc_moveSelectionPostButton.fill	= GridBagConstraints.BOTH;
		gbc_moveSelectionPostButton.gridx	= 2;
		gbc_moveSelectionPostButton.gridy	= 2;

		gbc_moveSelectionPostButton.insets = new Insets(10, 10, 10, 10);

		contentPane.add(postButton, gbc_moveSelectionPostButton);

		postButton.addActionListener((evt) -> {
			if (recipientID == 0) System.out.println("Please select recipient");

			// TODO: Acquire proper sender id
			if (!messageEnterTextfield.getText().isEmpty() && recipientID != 0) try {
				final Message message = client
					.createMessage(client.getSenderID(), recipientID, messageEnterTextfield.getText());
				client.sendMessage(message);
				Chat partnerChat = null;
				for (int i = 0; i < partnerChatList.size(); i++) {
					if (recipientID == partnerChatList.get(i).getUserID()) {
						partnerChat = partnerChatList.get(i);
						// System.out.println("partnerChatID: " + partnerChatList.get(i).getUserID());

					}
				}
				appendMessageToChat(message, partnerChat);
				messageEnterTextfield.setText("");
				elementList.setModel(partnerChat.getModel());
				contentPane.revalidate();
			} catch (Exception e) {
				JOptionPane.showMessageDialog(this,
						"An exception occured while sending a message. See the log for more details.",
						"Exception occured",
						JOptionPane.ERROR_MESSAGE);
				e.printStackTrace();
			}
		});

		// Partner name display
		JTextPane textPane = new JTextPane();
		textPane.setBackground(new Color(0, 0, 0));
		textPane.setForeground(new Color(255, 255, 255));

		textPane.setFont(new Font("Arial", Font.PLAIN, 20));

		GridBagConstraints gbc_partnerName = new GridBagConstraints();
		gbc_partnerName.fill		= GridBagConstraints.HORIZONTAL;
		gbc_partnerName.gridwidth	= 2;
		gbc_partnerName.gridx		= 1;
		gbc_partnerName.gridy		= 0;

		gbc_partnerName.insets = new Insets(0, 10, 0, 10);
		contentPane.add(textPane, gbc_partnerName);

		JList<User> userList = new JList<>();
		userList.setCellRenderer(new UserListRenderer());
		userList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		userList.addListSelectionListener((listSelectionEvent) -> {
			if (!listSelectionEvent.getValueIsAdjusting()) {
				@SuppressWarnings("unchecked")
				JList<User> selectedUserList = (JList<User>) listSelectionEvent.getSource();
				recipientID = selectedUserList.getModel().getElementAt(selectedUserList.getSelectedIndex()).getID();
				Chat selectedChat = null;

				for (int i = 0; i < partnerChatList.size(); i++) {
					if (selectedUserList.getSelectedIndex() == partnerChatList.get(i).getUserID() - 1) {
						selectedChat = partnerChatList.get(i);

					}
				}
				User partner = getPartner();
				System.out.println(partner.getName());
				textPane.setText(partner.getName());
				elementList.setModel(selectedChat.getModel());
				contentPane.revalidate();

			}
		});

		userList.setSelectionForeground(new Color(255, 255, 255));
		userList.setSelectionBackground(new Color(102, 0, 153));
		userList.setForeground(new Color(255, 255, 255));
		userList.setBackground(new Color(51, 51, 51));
		userList.setFont(new Font("Arial", Font.PLAIN, 17));
		userList.setBorder(new EmptyBorder(5, 5, 5, 5));

		GridBagConstraints gbc_userList = new GridBagConstraints();
		gbc_userList.fill = GridBagConstraints.VERTICAL;
		gbc_userList.gridx	= 0;
		gbc_userList.gridy	= 1;
		gbc_userList.anchor	= GridBagConstraints.PAGE_START;
		gbc_userList.insets	= new Insets(0, 0, 10, 0);

		contentPane.add(userList, gbc_userList);
		contentPane.revalidate();

		loadUserList(userList);
	}

	/**
	 * Initializes the elements of the user list by downloading them from the
	 * server.
	 * 
	 * @param userList The {@link JList} to put the elements in
	 */
	private void loadUserList(JList<User> userList) {
		new Thread(() -> {
			Users					users			= client.getUsersListXml();
			DefaultListModel<User>	userListModel	= new DefaultListModel<>();
			users.getUser().forEach(user -> userListModel.addElement(user));
			SwingUtilities.invokeLater(() -> userList.setModel(userListModel));
		}).start();
	}

	/**
	 * Extracts the first text content from a message.
	 * 
	 * @param message The message from which to return the text content
	 * @return The first content of type 'text'
	 */
	private String getFirstTextContent(Message message) { return message.getContent().get(0).getText(); }

	/**
	 * Appends a message with sender and message content to the message list.
	 * 
	 * @param message The message to append
	 */
	private void appendMessageToChat(Message message, Chat partnerChat) {
		partnerChat.addMessage("<html>"
				+ "<p style=\"color:#d2d235\"> <b> <small>"
				+ client.getClientUser().getName() + " "
				+ (message.getMetaData().getDate().getHour() + 2) + ":" //Dont know why this parameter 2 hours back.
				+ message.getMetaData().getDate().getMinute() + ":"
				+ message.getMetaData().getDate().getSecond()
				//Dont know
				+ "</b> </small>" + "<br>" + "<p style=\"color:white\">" + getFirstTextContent(message)
				+ "</span></html>");
		partnerChat.addMessageElement(message);
	}

	public User getPartner() {
		Users	users	= client.getUsersListXml();
		User	partner	= null;
		for (int i = 0; i < users.getUser().size(); i++) {
			if (users.getUser().get(i).getID() == recipientID) { partner = users.getUser().get(i); }
		}
		return partner;
	}
}